import java.util.*;

import iBus.*;
import iBus.layers.*;
import iBus.exception.*;

import Ticker.*;

/* ***************************************************************************
 * Classe: ConsumerVolatile
 *
 */
public class ConsumerVolatile {
  static String appName = "ConsumerVolatile";
  static Log log = new Log(appName);			// Objecto log
  static String channelVName = "Tick-Channel";
  static String channelVVer = "1.5";
  static String channelVSubject = "/P";
  static String channelVQoS = "Reliable";

  public static void main(String argv[]) throws Exception {
    Application.setAppName(appName);

    /* ***********************************************************************
     * Instancia a stack para o canal volatil (V).
     *
     */

    // Pilha de protocolos para difusao fiavel (reliable multicast)
    final iBus.Stack stackV = new iBus.Stack(channelVQoS);

    // Aumenta os valores do timeout e heartbeat para tornar a deteccao de
    // falhas menos agressiva (valores em milisegundos). Por outro lado,
    // assumimos que este canal nao perde mensagens.
    REACH reachV = (REACH)stackV.getProtocolObject("REACH");
    reachV.setTimeout(60000);
    reachV.setHbeat(10000);
    NAK nakV = (NAK)stackV.getProtocolObject("NAK");
    nakV.setHBInterval(60000);

    iBusURL url = null;			// Endereco do canal volatil (V)
    try {
      url = iBusURLFactory.create(channelVName, channelVVer, channelVSubject);
    }
    catch (Exception e) {
      // Coloca no log uma mensagem e termina a aplicacao (exit code=1)
      log.panic("main: iBusURLFactory.create failed: " + e);
    }


    /* ***********************************************************************
     * Define a aplicacao como um consumidor do canal V.
     *
     */

    // Instancia o objecto que recebe as mensagens enviadas em V
    ObjectReceiver rcv = new ObjectReceiver(log);

    // Conecta-se ao canal V como assinante (provoca uma mudanca de vista em V)
    try {
      stackV.subscribe(url, rcv);
    }
    catch (Exception e) {
      log.panic("main: stack.subscribe failed: " + e);
    }


    /* ***********************************************************************
     * Inicio da aplicacao
     *
     */
    System.out.println(Application.getAppName() + ": inicio da aplicacao.\n");

    stackV.waitTillExit();			// Aguarda por mensagens em V
  }
}



/* ***************************************************************************
 * Classe: ObjectReceiver
 *
 */
class ObjectReceiver implements iBus.Receiver {
  Log log;						// Objecto log
  static int count = 0;

  // Constructor
  public ObjectReceiver(Log log_) {
    log = log_;
  }

  // O metodo dispatchPush e' invocado quando uma mensagem e' recebida
  public void dispatchPush(iBusURL channelV, Posting posting) {
    for (int n = 0; n < posting.getLength(); n++) {
      String postingName = posting.getObject(0).getClass().getName();

      if (postingName.substring(0, 7).equals("Ticker.")) {
        System.out.println((count++) + "\t" + new Date().toString());
      }
      else {
        String str = (String)posting.getObject(n);
        System.out.println("Mensagem desconhecida: " + str.toString());
      }
    }

    System.out.flush();
  }

  // O metodo dispatchPull e' invocado quando uma mensagem e' enviada
  public Posting dispatchPull(iBusURL channelV, Posting request) {
    System.err.println("ObjectReceiver.dispatchPull called");
    return(null);
  }

  // O metodo error e' invocado quando ocorre um erro
  public void error(iBusURL channelV, String details) {
    log.panic("ObjectReceiver: internal error: " + details);
  }
}

// fim: ConsumerVolatil.java
